CREATED = "created"
UPDATED = "updated"
DELETED = "deleted"
PROMOTED = "promoted"
VIEWED = "viewed"
STATS = "stats"
CLONED = "cloned"
RESUMED = "resumed"
STARTED = "started"
RESTARTED = "restarted"
COPIED = "copied"
SUCCEEDED = "succeeded"
FAILED = "failed"
SYNCED = "synced"
DONE = "done"
STOPPED = "stopped"
APPROVED = "approved"
TRANSFERRED = "transferred"
INVALIDATED = "invalidated"
SKIPPED = "skipped"
SETTINGS = "settings"
BILLING = "billing"
SUPPORT_REQUESTED = "support_requested"
NEW_STATUS = "new_status"
NEW_STAGE = "new_stage"
NEW_ARTIFACTS = "new_artifacts"
ARCHIVED = "archived"
RESTORED = "restored"
GRANTED = "granted"
REVOKED = "revoked"
INVITED = "invited"
DENIED = "denied"
REGISTERED = "registered"
ACTIVATED = "activated"
DOWNLOADED = "downloaded"
BOOKMARKED = "bookmarked"
UNBOOKMARKED = "unbookmarked"
ARTIFACTS_DOWNLOADED = "artifacts_downloaded"
RESOURCES_VIEWED = "resources_viewed"
LOGS_VIEWED = "logs_viewed"
STATUSES_VIEWED = "statuses_viewed"
RUNS_VIEWED = "runs_viewed"
METRICS_VIEWED = "metrics_viewed"
PROJECTS_VIEWED = "projects_viewed"
EXECUTED = "executed"
AUTH = "auth"
LDAP = "ldap"
GITHUB = "github"
GITLAB = "gitlab"
BITBUCKET = "bitbucket"
GOOGLE = "google"


WRITE_ACTIONS = [
    CREATED,
    UPDATED,
    DELETED,
    RESUMED,
    RESTARTED,
    SYNCED,
    COPIED,
    CLONED,
    STOPPED,
    APPROVED,
    TRANSFERRED,
    SETTINGS,
    PROMOTED,
    INVALIDATED,
    BILLING,
    SUPPORT_REQUESTED,
    ARCHIVED,
    RESTORED,
    GRANTED,
    REVOKED,
    INVITED,
]
