ENABLED = "ENABLED"
URL = "URL"
GLOBAL_COUNTDOWN = "GLOBAL_COUNTDOWN"
DEFAULT_PREFIX = "DEFAULT_PREFIX"
TOKEN = "TOKEN"
CONTAINER = "CONTAINER"
BOOKMARKS = "BOOKMARKS"
ARCHIVES = "ARCHIVES"
DELETION = "DELETION"
SEARCHES = "SEARCHES"
IMAGE_PULL_POLICY = "IMAGE_PULL_POLICY"
DEFAULT_DOMAIN = "DEFAULT_DOMAIN"
NOTIFICATIONS = "NOTIFICATIONS"
ORGANIZATIONS = "ORGANIZATIONS"
ACTIVITY_LOGS = "ACTIVITY_LOGS"
HUB = "HUB"
