<!DOCTYPE html>
<html>

<head>
  {% load static %}
  <title>Polyaxon</title>
  <style>
    .plxLoading {
      position: fixed;
      top: 35%;
      left: 50%;
    }
    .plxLoading div {
      position: absolute;
      width: 16px;
      height: 16px;
      border-radius: 50%;
      background: #26303D;
      animation: plxLoading 1.2s linear infinite;
    }
    .plxLoading div:nth-child(1) {
      top: 8px;
      left: 8px;
      animation-delay: 0s;
    }
    .plxLoading div:nth-child(2) {
      top: 8px;
      left: 32px;
      animation-delay: -0.4s;
    }
    .plxLoading div:nth-child(3) {
      top: 32px;
      left: 8px;
      animation-delay: -0.4s;
    }
    .plxLoading div:nth-child(4) {
      top: 36px;
      left: 36px;
      animation-delay: -0.8s;
      width: 10px;
      height: 10px;
    }
    .plxLoading div:nth-child(5) {
      top: 56px;
      left: 8px;
      animation-delay: -0.8s;
    }
    .plxLoading div:nth-child(6) {
      top: 60px;
      left: 36px;
      animation-delay: -1.2s;
      width: 10px;
      height: 10px;
    }
    @keyframes plxLoading {
      0%, 100% {
        opacity: 1;
      }
      50% {
        opacity: 0.5;
      }
    }
  </style>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
  <link rel="icon" type="image/png" href="{% static 'images/favicon.ico' %}?version={{ ui_assets_version }}"/>
  <link rel="icon" type="image/svg+xml" href="{% static 'images/favicon.svg' %}?version={{ ui_assets_version }}"/>
  <base href="{{ ui_base_url }}">
</head>

<body class="plxBody">
  <div id="root" class="plxLayout">
    <div class="plxLoading"><div></div><div></div><div></div><div></div><div></div><div></div></div>
  </div>

  <!-- Dependencies -->
  {% if ui_enabled %}
  {% if ui_offline %}
  <script src="{% static 'vendors/js/react.production.18.0.2.min.js' %}"></script>
  <script src="{% static 'vendors/js/react-dom.production.18.0.2.min.js' %}"></script>
  <script src="{% static 'vendors/js/moment.2.29.3.min.js' %}"></script>
  <script src="{% static 'vendors/js/plotly.2.24.2.min.js' %}"></script>
  <script src="{% static 'vendors/js/create-plotly-component.2.6.0.min.js' %}"></script>
  <script src="{% static 'vendors/js/bokeh.3.2.0.min.js' %}"></script>
  <script src="{% static 'vendors/js/vega@5.min.js' %}"></script>
  <script src="{% static 'vendors/js/vega-lite@5.min.js' %}"></script>
  <script src="{% static 'vendors/js/vega-embed@6.min.js' %}"></script>
  {% else %}
  <script src="https://unpkg.com/react@18.2.0/umd/react.production.min.js"></script>
  <script src="https://unpkg.com/react-dom@18.2.0/umd/react-dom.production.min.js"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.29.3/moment.min.js"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/plotly.js/2.24.2/plotly.min.js"></script>
  <script src="https://unpkg.com/react-plotly.js@2.6.0/dist/create-plotly-component.min.js"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/bokeh/3.2.0/bokeh.min.js"></script>
  <script src="https://cdn.jsdelivr.net/npm/vega@5.25.0/build/vega.min.js"></script>
  <script src="https://cdn.jsdelivr.net/npm/vega-lite@5.11.1/build/vega-lite.min.js"></script>
  <script src="https://cdn.jsdelivr.net/npm/vega-embed@6.22.1/build/vega-embed.min.js"></script>
  {% endif %}
  <script src="{% static 'dist/js/0.bundle.js' %}?version={{ ui_assets_version }}"></script>
  <script src="{% static 'dist/js/1.bundle.js' %}?version={{ ui_assets_version }}"></script>
  <script src="{% static 'dist/js/2.bundle.js' %}?version={{ ui_assets_version }}"></script>
  <script src="{% static 'dist/js/3.bundle.js' %}?version={{ ui_assets_version }}"></script>
  <script src="{% static 'dist/js/4.bundle.js' %}?version={{ ui_assets_version }}"></script>
  <script src="{% static 'dist/js/5.bundle.js' %}?version={{ ui_assets_version }}"></script>
  <script src="{% static 'dist/js/6.bundle.js' %}?version={{ ui_assets_version }}"></script>
  <script src="{% static 'dist/js/7.bundle.js' %}?version={{ ui_assets_version }}"></script>
  {% endif %}
</body>

</html>
